import './App.css';
import React, {useEffect, useState} from "react";
import {
  Button,
  Container,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Grid,
  Pagination,
  Stack, Typography
} from "@mui/material";
import {makeStyles} from '@mui/styles';
import Package from "./components/Package";

const useStyles = makeStyles(() => ({
    paginator: {
        justifyContent: "center"
    }
}));

function App() {
    const classes = useStyles();
    const itemsPerPage = 100;
    const [data, setData] = useState([]);
    const [loading, setLoading] = useState(true);
    const [token, setToken] = useState('');
    const [popup, setPopup] = useState(null);
    const [activePage, setActivePage] = useState(1);

    useEffect(() => {
        let xmlHttp = new XMLHttpRequest();
        xmlHttp.open("GET", '/session/token', false);
        xmlHttp.send(null);
        setToken(xmlHttp.responseText);

        fetch("/package", {
            headers: {
                'Content-Type': 'application/json; charset=utf-8'
            }
        })
            .then(response => response.json())
            .then(json => setData(json))
            .then(() => setLoading(false))
            .catch((error) => console.error(error));
    }, []);

    useEffect(() => {
        if (activePage !== 1) {
            fetch(`/package?page=${activePage}`, {
              headers: {
                'Content-Type': 'application/json; charset=utf-8'
              }
            })
              .then(response => response.json())
              .then(json => setData(json))
              .then(() => setLoading(false))
              .catch((error) => console.error(error));
        }
    }, [activePage]);

    const handleChildClick = (response) => setPopup(response);

    const handlePageChange = (e, pageNumber) => {
      setLoading(true);setActivePage(pageNumber);
    }

    const renderPackageCard = () => {
        const pkgs = [];
        Object.entries(data?.packages).map(pkg => pkgs.push(pkg));

        return (
            <Container>
                <Stack spacing={2} mt={2}>
                    <Pagination
                        page={activePage}
                        count={data?.pages.last}
                        onChange={handlePageChange}
                        classes={{ul: classes.paginator}}
                    />
                </Stack>
                {
                    <Dialog
                        open={popup}
                        onClose={() => setPopup(false)}
                    >
                        <DialogTitle>
                            {popup?.message}
                        </DialogTitle>
                        <DialogContent>
                          <Typography
                            variant="body2"
                            sx={{fontSize: 13}}
                            dangerouslySetInnerHTML={{__html: popup?.composer_message}}
                          />
                        </DialogContent>
                        <DialogActions>
                            <Button autoFocus onClick={() => setPopup(false)}>
                                Got it
                            </Button>
                        </DialogActions>
                    </Dialog>
                }
              <Grid container spacing={2}>
                {pkgs.map((pkg, idx) => (
                  <Grid item md={6} xs={12} key={idx}>
                    <Package
                      pack={pkg[1]}
                      token={token}
                      onClickParent={response => handleChildClick(response)}
                    />
                  </Grid>
                ))}
              </Grid>
            </Container>
        )
    };

    return <>{!loading ? renderPackageCard() : 'Packages are on their way'}</>;
}

export default App;
