import React from 'react';
import {
    Box,
    Button,
    Card,
    CardContent,
    CardMedia,
    Chip,
    Grid,
    Link,
    Stack,
    Typography
} from "@mui/material";
import StarBorderPurple500Icon from '@mui/icons-material/StarBorderPurple500';
import VerifiedUserIcon from '@mui/icons-material/VerifiedUser';

const Package = ({pack, token, onClickParent}) => {
    let latest;
    if (pack.releases?.length) {
        latest = pack.releases?.reduce((max, rel) => max.date > rel.date ? max : rel);
    }

    const onClickHandler = (e) => {
        fetch('/package?_format=json', {
            method: 'POST',
            body: JSON.stringify({"package_name": e.target.dataset.pack}),
            headers: {
                'Content-Type': 'application/json; charset=utf-8',
                'X-CSRF-Token': token
            },
            credentials: 'include'
        })
          .then((response) => response.json())
          .then((json) => {
            onClickParent(json);
          })
    };

    return (
        <Card
            sx={{
                boxShadow: 1,
                borderRadius: 1,
                minWidth: 300,
                marginBottom: 2,
            }}
        >
            <CardContent>
                <Box sx={{display: 'flex', alignItems: 'flex-start', mb: 3}}>
                    <CardMedia
                        component="img"
                        sx={{width: 50}}
                        image={pack.image_url || 'assets/drupal-default-logo.svg'}
                        alt={pack.title}
                    />
                    <Box sx={{flexBasis: '65%', pl: 2, pr: 2}}>
                        <Box sx={{display: 'flex', alignItems: 'center'}}>
                            <Typography variant="h5" sx={{fontSize: 15, color: '#1565c0', fontWeight: 500}}>
                                {pack.title}
                            </Typography>
                            {!pack.has_issue_queue && <VerifiedUserIcon color="success"/>}
                        </Box>
                        <Typography
                            variant="body2"
                            sx={{fontSize: 13}}
                            dangerouslySetInnerHTML={{__html: pack.description}}
                        />
                        <Typography sx={{fontSize: 14}} gutterBottom>
                            By: <Link href={pack.author.uri}>
                            {pack.author.name}
                        </Link>
                        </Typography>
                    </Box>
                    <Button
                        variant="contained"
                        color="primary"
                        sx={{fontSize: 13}}
                        data-pack={pack.name}
                        onClick={onClickHandler}
                        disabled={!pack.isCompatible}
                    >
                        Download
                    </Button>
                </Box>
                <Box sx={{backgroundColor: 'grey.200', p: 1}}>
                    <Grid container spacing={2}>
                        <Grid item md={6}>
                            <Typography sx={{fontSize: 12, textTransform: 'capitalize'}}>
                                Security Coverage: {pack.security_coverage.replace('-', ' ')}
                            </Typography>
                            <Box sx={{display: 'flex', alignItems: 'center'}}>
                                <StarBorderPurple500Icon color="action" sx={{fontSize: 20, mr: 0.5}}/>
                                <Typography sx={{fontSize: 12}}>
                                    {pack.stars}
                                </Typography>
                            </Box>
                        </Grid>
                        <Grid item md={6}>
                            {pack.maintenanceStatus?.length ? (
                                <Typography sx={{fontSize: 12}}>
                                    Maintenance Status: {pack.maintenanceStatus}
                                </Typography>
                            ) : null
                            }
                            {pack.developmentStatus?.length ? (
                                <Typography sx={{fontSize: 12}}>
                                    Development Status: {pack.developmentStatus}
                                </Typography>
                            ) : null
                            }
                            {pack.releases?.length ? (
                                <>
                                    <Typography sx={{fontSize: 12}}>
                                        Latest Release: <Link href={latest.releaseLink}>{latest.version}</Link>
                                    </Typography>
                                    {latest.isCompatible ?
                                        (<Typography sx={{fontSize: 12}} color="success">
                                            Compatible with your Drupal installation
                                        </Typography>)
                                        : (<Typography sx={{fontSize: 12}} color="error">
                                            Not compatible with your Drupal installation
                                        </Typography>)
                                    }
                                </>
                            ) : null
                            }
                        </Grid>
                        <Grid item xs={12}>
                            <Stack direction="row" spacing={1} justifyContent="flex-start" flexWrap='wrap'>
                                {pack.categories?.length
                                    ? pack.categories.map((key, i) => {
                                        if (key) {
                                            return <Chip key={i} label={key} size="small" color="info"
                                                         sx={{fontSize: 10, marginBottom: 1}}/>
                                        }
                                        return false;
                                    })
                                    : null
                                }
                            </Stack>
                        </Grid>
                    </Grid>
                </Box>
            </CardContent>
        </Card>
    )
}

export default Package;
