<?php


namespace Drupal\drupal_marketplace\Entity;

use Drupal\Component\Utility\Unicode;

class Package {

  public string $title;
  public int $created;
  public int $changed;
  public string $name;
  public int $stars;
  public $url;
  public string $security_coverage;
  public $status;
  public $images;
  public $image_url;
  public $has_issue_queue;

  public string $description;

  public string $version;

  public array $releases;

  public string $maintenanceStatus;

  public string $developmentStatus;
  public array $categories;

  public bool $installed;
  public bool $isCompatible = FALSE;
  public $project_usage_count;
  public $taxonomy_vocabulary_46;
  public $taxonomy_vocabulary_44;

  public function __construct(array $project) {
    $module_list = \Drupal::moduleHandler()->getModuleList();
    $this->name = $project['field_project_machine_name'];
    $this->installed = array_key_exists($this->name, $module_list);
    $this->created = $project['created'];
    $this->changed = $project['changed'];
    $this->title = $project['title'];
    $this->setDescription($project['body']);
    $this->status = $project['status'];
    $this->stars = count($project['flag_project_star_user']);
    $this->security_coverage = $project['field_security_advisory_coverage'];
    $this->images = $project['field_project_images'];
    $this->url = $project['url'];
    $this->has_issue_queue = $project['field_project_has_issue_queue'];

    if (array_key_exists('author', $project)) {
      $this->author = $project['author'];
    }

    if (array_key_exists('project_usage', $project)) {
      $this->project_usage = $project['project_usage'];

      foreach ($this->project_usage as $usage) {
        $this->project_usage_count += $usage;
      }
    }

    // Module categories
    $this->taxonomy_vocabulary_3 = $project['taxonomy_vocabulary_3'];

    // Maintenance status
    if (array_key_exists('taxonomy_vocabulary_44', $project) && $project['taxonomy_vocabulary_44']) {
      $this->taxonomy_vocabulary_44 = $project['taxonomy_vocabulary_44'];
    }

    // Development status
    if (array_key_exists('taxonomy_vocabulary_46', $project) && $project['taxonomy_vocabulary_46']) {
      $this->taxonomy_vocabulary_46 = $project['taxonomy_vocabulary_46'];
    }
  }

  /**
   * @return string
   */
  public function getTitle(): string {
    return $this->title;
  }

  /**
   * @param string $title
   */
  public function setName(string $title): void {
    $this->title = $title;
  }

  /**
   * @return string
   */
  public function getDescription(): string {
    return $this->description;
  }

  /**
   * @param array $body
   */
  public function setDescription(array $body): void {
    if (!array_key_exists('value', $body)) {
      $this->description = '';
    }
    else {
      $this->description = strip_tags($body['value'], [
        'p',
        'a',
        'b',
        'i',
        'u',
        'em',
        'strong'
      ]);
      $this->description = Unicode::truncate($this->description, 200, TRUE, TRUE);
    }
  }

  /**
   * @return string
   */
  public function getVersion(): string {
    return $this->version;
  }

  /**
   * @param string $version
   */
  public function setVersion(string $version): void {
    $this->version = $version;
  }

  public function setReleases($releases) {
    $this->releases = array_map(function ($release) {
      $release_object = new Release($release);
      if (!$this->isCompatible && $release_object->isCompatible) {
        $this->isCompatible = TRUE;
      }
      return $release_object;
    }, $releases);
  }

  public function getReleases(): array {
    return $this->releases;
  }

  public function getLatestRelease() {
    // @todo need to understand logic of the array from d.org and fix this method.
    return reset($this->releases);
  }

  public function setImageUrl(string $url) {
    $this->image_url = $url;
  }

  public function setMaintenanseStatus(string $status) {
    $this->maintenanceStatus = $status;
  }

  public function getMaintenanseStatus() {
    return $this->maintenanceStatus;
  }

  public function setDevelopmentStatus(string $status) {
    $this->developmentStatus = $status;
  }

  public function getDevelopmentStatus(): string {
    return $this->developmentStatus;
  }

  public function setCategories(array $categories) {
    $this->categories = $categories;
  }

  public function getCategoriesStatus(): array {
    return $this->categories;
  }

  public function setCompatibility(bool $compatibility) {
    $this->isCompatible = $compatibility;
  }


}
