<?php

namespace Drupal\drupal_marketplace\Service;

use Composer\Console\Application;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Output\StreamOutput;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Class ComposerWrapperService.
 */
class ComposerWrapper {

  /**
   * The console application that handles the commands.
   *
   * @var \Composer\Console\Application
   */
  protected $application;

  /**
   * Stream resource.
   *
   * @var false|resource
   */
  protected $stream;

  protected $eventDispatcher;

  /**
   * ComposerWrapperService constructor.
   */
  public function __construct(EventDispatcherInterface $eventDispatcher) {
    $this->initComposer();
    $this->stream = fopen('php://temp', 'w+');
    $this->eventDispatcher = $eventDispatcher;
  }

  /**
   * Init composer.
   */
  private function initComposer() {
    $this->application = new Application();
    $this->application->setAutoExit(FALSE);
  }

  /**
   * Create output stream.
   *
   * @return \Symfony\Component\Console\Output\StreamOutput
   *   Stream.
   */
  private function streamOutput() {
    chdir(__DIR__);

    return new StreamOutput($this->stream);
  }

  /**
   * Search command.
   *
   * @param string $package
   *   Package name.
   *
   * @return array
   *   Results.
   */
  public function searchCommand(string $package): array {
    try {
      $output = $this->streamOutput();

      $code = $this->application->run(new ArrayInput([
        'command' => 'search',
        'tokens' => [$package],
        '--working-dir' => $this->getWorkingDir(),
      ]), $output);
    } catch (\Exception $e) {
      $message = $e->getMessage();
      // TODO handle logic.
    }
    rewind($this->stream);
    $result = stream_get_contents($this->stream);
    return [
      'code' => $code,
      'message' => $result,
    ];
  }

  /**
   * Remove command.
   *
   * @param string $package
   *   Package name.
   *
   * @return array
   *   Results.
   */
  public function removeCommand(string $package): array {
    try {
      $output = $this->streamOutput();

      $code = $this->application->run(new ArrayInput([
        'command' => 'remove',
        'packages' => [$package],
        '--working-dir' => $this->getWorkingDir(),
      ]), $output);
    } catch (\Exception $e) {
      $message = $e->getMessage();
      // TODO handle logic.
    }
    rewind($this->stream);
    $result = stream_get_contents($this->stream);
    return [
      'code' => $code,
      'message' => $result,
    ];
  }

  /**
   * Require command.
   *
   * @param string $package
   *   Package name.
   */
  public function requireCommand(string $package): array {
    try {
      $output = $this->streamOutput();

      $code = $this->application->run(new ArrayInput([
        'command' => 'require',
        'packages' => [$package],
        '--working-dir' => $this->getWorkingDir(),
      ]), $output);
    } catch (\Exception $e) {
      $message = $e->getMessage();
      // TODO handle logic.
    }
    rewind($this->stream);
    $result = stream_get_contents($this->stream);
    return [
      'code' => $code,
      'message' => $result,
    ];
  }

  /**
   * Get working dir.
   */
  private function getWorkingDir(): string {
    return DRUPAL_ROOT . '/..';
  }

}
