<?php

namespace Drupal\drupal_marketplace\Service;

use Doctrine\Common\Cache\FilesystemCache;
use GuzzleHttp\Client;
use GuzzleHttp\HandlerStack;
use Kevinrob\GuzzleCache\CacheMiddleware;
use Kevinrob\GuzzleCache\Storage\DoctrineCacheStorage;
use Kevinrob\GuzzleCache\Strategy\PrivateCacheStrategy;

class HttpClient {

  protected $guzzleClient;

  /**
   * Getter for the Guzzle client.
   *
   * @return \GuzzleHttp\Client
   *   The HTTP Client.
   */
  public function getGuzzleClient(): Client {
    if (!isset($this->guzzleClient)) {
      $this->setGuzzleClient($this->createGuzzleClient());
    }
    return $this->guzzleClient;
  }

  /**
   * Setter for the Guzzle client.
   *
   * @param \GuzzleHttp\Client $client
   *   The Guzzle client.
   */
  public function setGuzzleClient(Client $client): void {
    $this->guzzleClient = $client;
  }

  /**
   * Creates a Guzzle client with local file caching middleware.
   *
   * @return \GuzzleHttp\Client
   *   The Guzzle client.
   */
  protected function createGuzzleClient(): Client {
    $stack = HandlerStack::create();
    $stack->push(
      new CacheMiddleware(new PrivateCacheStrategy(new DoctrineCacheStorage(new FilesystemCache(sys_get_temp_dir())))),
      'cache'
    );

    return new Client(['handler' => $stack]);
  }

}
